@@ -7,10 +7,12 @@ import matplotlib.colors as mcolors
 from matplotlib import rc_context
 from matplotlib.testing.decorators import image_comparison
 import matplotlib.pyplot as plt
-from matplotlib.colors import (BoundaryNorm, LogNorm, PowerNorm, Normalize,
-                               TwoSlopeNorm)
-from matplotlib.colorbar import ColorbarBase, _ColorbarLogLocator
+from matplotlib.colors import (
+    BoundaryNorm, LogNorm, PowerNorm, Normalize, NoNorm
+)
+from matplotlib.colorbar import Colorbar
 from matplotlib.ticker import FixedLocator
+from matplotlib.testing.decorators import check_figures_equal
 
 
 def _get_cmap_norms():
@@ -21,10 +23,10 @@ def _get_cmap_norms():
     Helper function for _colorbar_extension_shape and
     colorbar_extension_length.
     """
-    # Create a color map and specify the levels it represents.
+    # Create a colormap and specify the levels it represents.
     cmap = cm.get_cmap("RdBu", lut=5)
     clevs = [-5., -2.5, -.5, .5, 1.5, 3.5]
-    # Define norms for the color maps.
+    # Define norms for the colormaps.
     norms = dict()
     norms['neither'] = BoundaryNorm(clevs, len(clevs) - 1)
     norms['min'] = BoundaryNorm([-10] + clevs[1:], len(clevs) - 1)
@@ -54,10 +56,10 @@ def _colorbar_extension_shape(spacing):
         # Create a subplot.
         cax = fig.add_subplot(4, 1, i + 1)
         # Generate the colorbar.
-        ColorbarBase(cax, cmap=cmap, norm=norm,
-                     boundaries=boundaries, values=values,
-                     extend=extension_type, extendrect=True,
-                     orientation='horizontal', spacing=spacing)
+        Colorbar(cax, cmap=cmap, norm=norm,
+                 boundaries=boundaries, values=values,
+                 extend=extension_type, extendrect=True,
+                 orientation='horizontal', spacing=spacing)
         # Turn off text and ticks.
         cax.tick_params(left=False, labelleft=False,
                         bottom=False, labelbottom=False)
@@ -86,13 +88,13 @@ def _colorbar_extension_length(spacing):
             # Create a subplot.
             cax = fig.add_subplot(12, 1, i*3 + j + 1)
             # Generate the colorbar.
-            ColorbarBase(cax, cmap=cmap, norm=norm,
-                         boundaries=boundaries, values=values,
-                         extend=extension_type, extendfrac=extendfrac,
-                         orientation='horizontal', spacing=spacing)
+            Colorbar(cax, cmap=cmap, norm=norm,
+                     boundaries=boundaries, values=values,
+                     extend=extension_type, extendfrac=extendfrac,
+                     orientation='horizontal', spacing=spacing)
             # Turn off text and ticks.
             cax.tick_params(left=False, labelleft=False,
-                            bottom=False, labelbottom=False)
+                              bottom=False, labelbottom=False)
     # Return the figure to the caller.
     return fig
 
@@ -101,20 +103,28 @@ def _colorbar_extension_length(spacing):
                    'colorbar_extensions_shape_proportional.png'])
 def test_colorbar_extension_shape():
     """Test rectangular colorbar extensions."""
+    # Remove this line when this test image is regenerated.
+    plt.rcParams['pcolormesh.snap'] = False
+
     # Create figures for uniform and proportionally spaced colorbars.
     _colorbar_extension_shape('uniform')
     _colorbar_extension_shape('proportional')
 
 
 @image_comparison(['colorbar_extensions_uniform.png',
-                   'colorbar_extensions_proportional.png'])
+                   'colorbar_extensions_proportional.png'],
+                  tol=1.0)
 def test_colorbar_extension_length():
     """Test variable length colorbar extensions."""
+    # Remove this line when this test image is regenerated.
+    plt.rcParams['pcolormesh.snap'] = False
+
     # Create figures for uniform and proportionally spaced colorbars.
     _colorbar_extension_length('uniform')
     _colorbar_extension_length('proportional')
 
 
+@pytest.mark.parametrize('use_gridspec', [True, False])
 @image_comparison(['cbar_with_orientation',
                    'cbar_locationing',
                    'double_cbar',
@@ -122,21 +132,24 @@ def test_colorbar_extension_length():
                    ],
                   extensions=['png'], remove_text=True,
                   savefig_kwarg={'dpi': 40})
-def test_colorbar_positioning():
+def test_colorbar_positioning(use_gridspec):
+    # Remove this line when this test image is regenerated.
+    plt.rcParams['pcolormesh.snap'] = False
+
     data = np.arange(1200).reshape(30, 40)
     levels = [0, 200, 400, 600, 800, 1000, 1200]
 
     # -------------------
     plt.figure()
     plt.contourf(data, levels=levels)
-    plt.colorbar(orientation='horizontal', use_gridspec=False)
+    plt.colorbar(orientation='horizontal', use_gridspec=use_gridspec)
 
     locations = ['left', 'right', 'top', 'bottom']
     plt.figure()
     for i, location in enumerate(locations):
         plt.subplot(2, 2, i + 1)
         plt.contourf(data, levels=levels)
-        plt.colorbar(location=location, use_gridspec=False)
+        plt.colorbar(location=location, use_gridspec=use_gridspec)
 
     # -------------------
     plt.figure()
@@ -152,9 +165,9 @@ def test_colorbar_positioning():
     plt.contour(hatch_mappable, colors='black')
 
     plt.colorbar(color_mappable, location='left', label='variable 1',
-                 use_gridspec=False)
+                 use_gridspec=use_gridspec)
     plt.colorbar(hatch_mappable, location='right', label='variable 2',
-                 use_gridspec=False)
+                 use_gridspec=use_gridspec)
 
     # -------------------
     plt.figure()
@@ -166,11 +179,22 @@ def test_colorbar_positioning():
     plt.contourf(data, levels=levels)
 
     plt.colorbar(ax=[ax2, ax3, ax1], location='right', pad=0.0, shrink=0.5,
-                 panchor=False, use_gridspec=False)
+                 panchor=False, use_gridspec=use_gridspec)
     plt.colorbar(ax=[ax2, ax3, ax1], location='left', shrink=0.5,
-                 panchor=False, use_gridspec=False)
+                 panchor=False, use_gridspec=use_gridspec)
     plt.colorbar(ax=[ax1], location='bottom', panchor=False,
-                 anchor=(0.8, 0.5), shrink=0.6, use_gridspec=False)
+                 anchor=(0.8, 0.5), shrink=0.6, use_gridspec=use_gridspec)
+
+
+@image_comparison(['contour_colorbar.png'], remove_text=True)
+def test_contour_colorbar():
+    fig, ax = plt.subplots(figsize=(4, 2))
+    data = np.arange(1200).reshape(30, 40) - 500
+    levels = np.array([0, 200, 400, 600, 800, 1000, 1200]) - 500
+
+    CS = ax.contour(data, levels=levels, extend='both')
+    fig.colorbar(CS, orientation='horizontal', extend='both')
+    fig.colorbar(CS, orientation='vertical')
 
 
 @image_comparison(['cbar_with_subplots_adjust.png'], remove_text=True,
@@ -209,28 +233,48 @@ def test_colorbar_single_scatter():
                          ids=['no gridspec', 'with gridspec'])
 def test_remove_from_figure(use_gridspec):
     """
-    Test `remove_from_figure` with the specified ``use_gridspec`` setting
+    Test `remove` with the specified ``use_gridspec`` setting
     """
     fig, ax = plt.subplots()
     sc = ax.scatter([1, 2], [3, 4], cmap="spring")
     sc.set_array(np.array([5, 6]))
-    pre_figbox = np.array(ax.figbox)
+    pre_position = ax.get_position()
     cb = fig.colorbar(sc, use_gridspec=use_gridspec)
     fig.subplots_adjust()
     cb.remove()
     fig.subplots_adjust()
-    post_figbox = np.array(ax.figbox)
-    assert (pre_figbox == post_figbox).all()
+    post_position = ax.get_position()
+    assert (pre_position.get_points() == post_position.get_points()).all()
+
+
+def test_remove_from_figure_cl():
+    """
+    Test `remove` with constrained_layout
+    """
+    fig, ax = plt.subplots(constrained_layout=True)
+    sc = ax.scatter([1, 2], [3, 4], cmap="spring")
+    sc.set_array(np.array([5, 6]))
+    fig.draw_without_rendering()
+    pre_position = ax.get_position()
+    cb = fig.colorbar(sc)
+    cb.remove()
+    fig.draw_without_rendering()
+    post_position = ax.get_position()
+    np.testing.assert_allclose(pre_position.get_points(),
+                               post_position.get_points())
 
 
 def test_colorbarbase():
     # smoke test from #3805
     ax = plt.gca()
-    ColorbarBase(ax, cmap=plt.cm.bone)
+    Colorbar(ax, cmap=plt.cm.bone)
 
 
-@image_comparison(['colorbar_closed_patch'], remove_text=True)
+@image_comparison(['colorbar_closed_patch.png'], remove_text=True)
 def test_colorbar_closed_patch():
+    # Remove this line when this test image is regenerated.
+    plt.rcParams['pcolormesh.snap'] = False
+
     fig = plt.figure(figsize=(8, 6))
     ax1 = fig.add_axes([0.05, 0.85, 0.9, 0.1])
     ax2 = fig.add_axes([0.1, 0.65, 0.75, 0.1])
@@ -296,8 +340,8 @@ def test_colorbar_minorticks_on_off():
         cbar.minorticks_on()
         np.testing.assert_almost_equal(
             cbar.ax.yaxis.get_minorticklocs(),
-            [-1.2, -1.1, -0.9, -0.8, -0.7, -0.6, -0.4, -0.3, -0.2, -0.1,
-             0.1, 0.2, 0.3, 0.4, 0.6, 0.7, 0.8, 0.9, 1.1, 1.2])
+            [-1.1, -0.9, -0.8, -0.7, -0.6, -0.4, -0.3, -0.2, -0.1,
+             0.1, 0.2, 0.3, 0.4, 0.6, 0.7, 0.8, 0.9, 1.1, 1.2, 1.3])
 
     # tests for github issue #13257 and PR #13265
     data = np.random.uniform(low=1, high=10, size=(20, 20))
@@ -305,8 +349,8 @@ def test_colorbar_minorticks_on_off():
     fig, ax = plt.subplots()
     im = ax.pcolormesh(data, norm=LogNorm())
     cbar = fig.colorbar(im)
+    fig.canvas.draw()
     default_minorticklocks = cbar.ax.yaxis.get_minorticklocs()
-
     # test that minorticks turn off for LogNorm
     cbar.minorticks_off()
     np.testing.assert_equal(cbar.ax.yaxis.get_minorticklocs(), [])
@@ -367,10 +411,12 @@ def test_colorbar_autoticks():
         pcm = ax[1].pcolormesh(X, Y, Z)
         cbar2 = fig.colorbar(pcm, ax=ax[1], extend='both',
                              orientation='vertical', shrink=0.4)
+        # note only -10 to 10 are visible,
         np.testing.assert_almost_equal(cbar.ax.yaxis.get_ticklocs(),
-                                       np.arange(-10, 11, 5))
+                                       np.arange(-15, 16, 5))
+        # note only -10 to 10 are visible
         np.testing.assert_almost_equal(cbar2.ax.yaxis.get_ticklocs(),
-                                       np.arange(-10, 11, 10))
+                                       np.arange(-20, 21, 10))
 
 
 def test_colorbar_autotickslog():
@@ -389,10 +435,12 @@ def test_colorbar_autotickslog():
         pcm = ax[1].pcolormesh(X, Y, 10**Z, norm=LogNorm())
         cbar2 = fig.colorbar(pcm, ax=ax[1], extend='both',
                              orientation='vertical', shrink=0.4)
+        # note only -12 to +12 are visible
         np.testing.assert_almost_equal(cbar.ax.yaxis.get_ticklocs(),
-                                       10**np.arange(-12., 12.2, 4.))
+                                       10**np.arange(-16., 16.2, 4.))
+        # note only -24 to +24 are visible
         np.testing.assert_almost_equal(cbar2.ax.yaxis.get_ticklocs(),
-                                       10**np.arange(-12., 13., 12.))
+                                       10**np.arange(-24., 25., 12.))
 
 
 def test_colorbar_get_ticks():
@@ -412,27 +460,44 @@ def test_colorbar_get_ticks():
     assert userTicks.get_ticks().tolist() == [600, 700, 800]
 
     # testing for getter after calling set_ticks with some ticks out of bounds
-    userTicks.set_ticks([600, 1300, 1400, 1500])
-    assert userTicks.get_ticks().tolist() == [600]
+    # removed #20054: other axes don't trim fixed lists, so colorbars
+    # should not either:
+    # userTicks.set_ticks([600, 1300, 1400, 1500])
+    # assert userTicks.get_ticks().tolist() == [600]
 
     # testing getter when no ticks are assigned
     defTicks = plt.colorbar(orientation='horizontal')
-    assert defTicks.get_ticks().tolist() == levels
+    np.testing.assert_allclose(defTicks.get_ticks().tolist(), levels)
+
+    # test normal ticks and minor ticks
+    fig, ax = plt.subplots()
+    x = np.arange(-3.0, 4.001)
+    y = np.arange(-4.0, 3.001)
+    X, Y = np.meshgrid(x, y)
+    Z = X * Y
+    Z = Z[:-1, :-1]
+    pcm = ax.pcolormesh(X, Y, Z)
+    cbar = fig.colorbar(pcm, ax=ax, extend='both',
+                        orientation='vertical')
+    ticks = cbar.get_ticks()
+    np.testing.assert_allclose(ticks, np.arange(-15, 16, 5))
+    assert len(cbar.get_ticks(minor=True)) == 0
 
 
-def test_colorbar_lognorm_extension():
+@pytest.mark.parametrize("extend", ['both', 'min', 'max'])
+def test_colorbar_lognorm_extension(extend):
     # Test that colorbar with lognorm is extended correctly
     f, ax = plt.subplots()
-    cb = ColorbarBase(ax, norm=LogNorm(vmin=0.1, vmax=1000.0),
-                      orientation='vertical', extend='both')
+    cb = Colorbar(ax, norm=LogNorm(vmin=0.1, vmax=1000.0),
+                  orientation='vertical', extend=extend)
     assert cb._values[0] >= 0.0
 
 
 def test_colorbar_powernorm_extension():
     # Test that colorbar with powernorm is extended correctly
     f, ax = plt.subplots()
-    cb = ColorbarBase(ax, norm=PowerNorm(gamma=0.5, vmin=0.0, vmax=1.0),
-                      orientation='vertical', extend='both')
+    cb = Colorbar(ax, norm=PowerNorm(gamma=0.5, vmin=0.0, vmax=1.0),
+                  orientation='vertical', extend='both')
     assert cb._values[0] >= 0.0
 
 
@@ -451,13 +516,13 @@ def test_colorbar_log_minortick_labels():
         pcm = ax.imshow([[10000, 50000]], norm=LogNorm())
         cb = fig.colorbar(pcm)
         fig.canvas.draw()
-        lb = cb.ax.yaxis.get_ticklabels(which='both')
+        lb = [l.get_text() for l in cb.ax.yaxis.get_ticklabels(which='both')]
         expected = [r'$\mathdefault{10^{4}}$',
                     r'$\mathdefault{2\times10^{4}}$',
                     r'$\mathdefault{3\times10^{4}}$',
                     r'$\mathdefault{4\times10^{4}}$']
-        for l, exp in zip(lb, expected):
-            assert l.get_text() == exp
+        for exp in expected:
+            assert exp in lb
 
 
 def test_colorbar_renorm():
@@ -468,16 +533,15 @@ def test_colorbar_renorm():
     im = ax.imshow(z)
     cbar = fig.colorbar(im)
     np.testing.assert_allclose(cbar.ax.yaxis.get_majorticklocs(),
-                               np.arange(0, 120000.1, 15000))
+                               np.arange(0, 120000.1, 20000))
 
     cbar.set_ticks([1, 2, 3])
     assert isinstance(cbar.locator, FixedLocator)
 
     norm = LogNorm(z.min(), z.max())
     im.set_norm(norm)
-    assert isinstance(cbar.locator, _ColorbarLogLocator)
     np.testing.assert_allclose(cbar.ax.yaxis.get_majorticklocs(),
-                               np.logspace(-8, 5, 14))
+                               np.logspace(-10, 7, 18))
     # note that set_norm removes the FixedLocator...
     assert np.isclose(cbar.vmin, z.min())
     cbar.set_ticks([1, 2, 3])
@@ -500,19 +564,19 @@ def test_colorbar_format():
     im = ax.imshow(z)
     cbar = fig.colorbar(im, format='%4.2e')
     fig.canvas.draw()
-    assert cbar.ax.yaxis.get_ticklabels()[4].get_text() == '6.00e+04'
+    assert cbar.ax.yaxis.get_ticklabels()[4].get_text() == '8.00e+04'
 
     # make sure that if we change the clim of the mappable that the
     # formatting is *not* lost:
     im.set_clim([4, 200])
     fig.canvas.draw()
-    assert cbar.ax.yaxis.get_ticklabels()[4].get_text() == '8.00e+01'
+    assert cbar.ax.yaxis.get_ticklabels()[4].get_text() == '2.00e+02'
 
     # but if we change the norm:
     im.set_norm(LogNorm(vmin=0.1, vmax=10))
     fig.canvas.draw()
     assert (cbar.ax.yaxis.get_ticklabels()[0].get_text() ==
-            r'$\mathdefault{10^{-1}}$')
+            r'$\mathdefault{10^{-2}}$')
 
 
 def test_colorbar_scale_reset():
@@ -538,7 +602,7 @@ def test_colorbar_get_ticks_2():
     fig, ax = plt.subplots()
     pc = ax.pcolormesh([[.05, .95]])
     cb = fig.colorbar(pc)
-    np.testing.assert_allclose(cb.get_ticks(), [0.2, 0.4, 0.6, 0.8])
+    np.testing.assert_allclose(cb.get_ticks(), [0., 0.2, 0.4, 0.6, 0.8, 1.0])
 
 
 def test_colorbar_inverted_ticks():
@@ -561,16 +625,6 @@ def test_colorbar_inverted_ticks():
     np.testing.assert_allclose(minorticks, cbar.get_ticks(minor=True))
 
 
-def test_extend_colorbar_customnorm():
-    # This was a funny error with TwoSlopeNorm, maybe with other norms,
-    # when extend='both'
-    fig, (ax0, ax1) = plt.subplots(2, 1)
-    pcm = ax0.pcolormesh([[0]], norm=TwoSlopeNorm(vcenter=0., vmin=-2, vmax=1))
-    cb = fig.colorbar(pcm, ax=ax0, extend='both')
-    np.testing.assert_allclose(cb.ax.get_position().extents,
-                               [0.78375, 0.536364, 0.796147, 0.9], rtol=1e-3)
-
-
 def test_mappable_no_alpha():
     fig, ax = plt.subplots()
     sm = cm.ScalarMappable(norm=mcolors.Normalize(), cmap='viridis')
@@ -579,6 +633,18 @@ def test_mappable_no_alpha():
     plt.draw()
 
 
+def test_mappable_2d_alpha():
+    fig, ax = plt.subplots()
+    x = np.arange(1, 5).reshape(2, 2)/4
+    pc = ax.pcolormesh(x, alpha=x)
+    cb = fig.colorbar(pc, ax=ax)
+    # The colorbar's alpha should be None and the mappable should still have
+    # the original alpha array
+    assert cb.alpha is None
+    assert pc.get_alpha() is x
+    fig.draw_without_rendering()
+
+
 def test_colorbar_label():
     """
     Test the label parameter. It should just be mapped to the xlabel/ylabel of
@@ -609,3 +675,264 @@ def test_colorbar_int(clim):
     im = ax.imshow([[*map(np.int16, clim)]])
     fig.colorbar(im)
     assert (im.norm.vmin, im.norm.vmax) == clim
+
+
+def test_anchored_cbar_position_using_specgrid():
+    data = np.arange(1200).reshape(30, 40)
+    levels = [0, 200, 400, 600, 800, 1000, 1200]
+    shrink = 0.5
+    anchor_y = 0.3
+    # right
+    fig, ax = plt.subplots()
+    cs = ax.contourf(data, levels=levels)
+    cbar = plt.colorbar(
+            cs, ax=ax, use_gridspec=True,
+            location='right', anchor=(1, anchor_y), shrink=shrink)
+
+    # the bottom left corner of one ax is (x0, y0)
+    # the top right corner of one ax is (x1, y1)
+    # p0: the vertical / horizontal position of anchor
+    x0, y0, x1, y1 = ax.get_position().extents
+    cx0, cy0, cx1, cy1 = cbar.ax.get_position().extents
+    p0 = (y1 - y0) * anchor_y + y0
+
+    np.testing.assert_allclose(
+            [cy1, cy0],
+            [y1 * shrink + (1 - shrink) * p0, p0 * (1 - shrink) + y0 * shrink])
+
+    # left
+    fig, ax = plt.subplots()
+    cs = ax.contourf(data, levels=levels)
+    cbar = plt.colorbar(
+            cs, ax=ax, use_gridspec=True,
+            location='left', anchor=(1, anchor_y), shrink=shrink)
+
+    # the bottom left corner of one ax is (x0, y0)
+    # the top right corner of one ax is (x1, y1)
+    # p0: the vertical / horizontal position of anchor
+    x0, y0, x1, y1 = ax.get_position().extents
+    cx0, cy0, cx1, cy1 = cbar.ax.get_position().extents
+    p0 = (y1 - y0) * anchor_y + y0
+
+    np.testing.assert_allclose(
+            [cy1, cy0],
+            [y1 * shrink + (1 - shrink) * p0, p0 * (1 - shrink) + y0 * shrink])
+
+    # top
+    shrink = 0.5
+    anchor_x = 0.3
+    fig, ax = plt.subplots()
+    cs = ax.contourf(data, levels=levels)
+    cbar = plt.colorbar(
+            cs, ax=ax, use_gridspec=True,
+            location='top', anchor=(anchor_x, 1), shrink=shrink)
+
+    # the bottom left corner of one ax is (x0, y0)
+    # the top right corner of one ax is (x1, y1)
+    # p0: the vertical / horizontal position of anchor
+    x0, y0, x1, y1 = ax.get_position().extents
+    cx0, cy0, cx1, cy1 = cbar.ax.get_position().extents
+    p0 = (x1 - x0) * anchor_x + x0
+
+    np.testing.assert_allclose(
+            [cx1, cx0],
+            [x1 * shrink + (1 - shrink) * p0, p0 * (1 - shrink) + x0 * shrink])
+
+    # bottom
+    shrink = 0.5
+    anchor_x = 0.3
+    fig, ax = plt.subplots()
+    cs = ax.contourf(data, levels=levels)
+    cbar = plt.colorbar(
+            cs, ax=ax, use_gridspec=True,
+            location='bottom', anchor=(anchor_x, 1), shrink=shrink)
+
+    # the bottom left corner of one ax is (x0, y0)
+    # the top right corner of one ax is (x1, y1)
+    # p0: the vertical / horizontal position of anchor
+    x0, y0, x1, y1 = ax.get_position().extents
+    cx0, cy0, cx1, cy1 = cbar.ax.get_position().extents
+    p0 = (x1 - x0) * anchor_x + x0
+
+    np.testing.assert_allclose(
+            [cx1, cx0],
+            [x1 * shrink + (1 - shrink) * p0, p0 * (1 - shrink) + x0 * shrink])
+
+
+@image_comparison(['colorbar_change_lim_scale.png'], remove_text=True,
+                  style='mpl20')
+def test_colorbar_change_lim_scale():
+    fig, ax = plt.subplots(1, 2, constrained_layout=True)
+    pc = ax[0].pcolormesh(np.arange(100).reshape(10, 10)+1)
+    cb = fig.colorbar(pc, ax=ax[0], extend='both')
+    cb.ax.set_yscale('log')
+
+    pc = ax[1].pcolormesh(np.arange(100).reshape(10, 10)+1)
+    cb = fig.colorbar(pc, ax=ax[1], extend='both')
+    cb.ax.set_ylim([20, 90])
+
+
+@check_figures_equal(extensions=["png"])
+def test_axes_handles_same_functions(fig_ref, fig_test):
+    # prove that cax and cb.ax are functionally the same
+    for nn, fig in enumerate([fig_ref, fig_test]):
+        ax = fig.add_subplot()
+        pc = ax.pcolormesh(np.ones(300).reshape(10, 30))
+        cax = fig.add_axes([0.9, 0.1, 0.03, 0.8])
+        cb = fig.colorbar(pc, cax=cax)
+        if nn == 0:
+            caxx = cax
+        else:
+            caxx = cb.ax
+        caxx.set_yticks(np.arange(0, 20))
+        caxx.set_yscale('log')
+        caxx.set_position([0.92, 0.1, 0.02, 0.7])
+
+
+def test_inset_colorbar_layout():
+    fig, ax = plt.subplots(constrained_layout=True, figsize=(3, 6))
+    pc = ax.imshow(np.arange(100).reshape(10, 10))
+    cax = ax.inset_axes([1.02, 0.1, 0.03, 0.8])
+    cb = fig.colorbar(pc, cax=cax)
+
+    fig.draw_without_rendering()
+    # make sure this is in the figure. In the colorbar swapping
+    # it was being dropped from the list of children...
+    np.testing.assert_allclose(cb.ax.get_position().bounds,
+                               [0.87, 0.342, 0.0237, 0.315], atol=0.01)
+    assert cb.ax in ax.child_axes
+
+
+@image_comparison(['colorbar_twoslope.png'], remove_text=True,
+                  style='mpl20')
+def test_twoslope_colorbar():
+    # Note that the second tick = 20, and should be in the middle
+    # of the colorbar (white)
+    # There should be no tick right at the bottom, nor at the top.
+    fig, ax = plt.subplots()
+
+    norm = mcolors.TwoSlopeNorm(20, 5, 95)
+    pc = ax.pcolormesh(np.arange(1, 11), np.arange(1, 11),
+                       np.arange(100).reshape(10, 10),
+                       norm=norm, cmap='RdBu_r')
+    fig.colorbar(pc)
+
+
+@check_figures_equal(extensions=["png"])
+def test_remove_cb_whose_mappable_has_no_figure(fig_ref, fig_test):
+    ax = fig_test.add_subplot()
+    cb = fig_test.colorbar(cm.ScalarMappable(), cax=ax)
+    cb.remove()
+
+
+def test_aspects():
+    fig, ax = plt.subplots(3, 2, figsize=(8, 8))
+    aspects = [20, 20, 10]
+    extends = ['neither', 'both', 'both']
+    cb = [[None, None, None], [None, None, None]]
+    for nn, orient in enumerate(['vertical', 'horizontal']):
+        for mm, (aspect, extend) in enumerate(zip(aspects, extends)):
+            pc = ax[mm, nn].pcolormesh(np.arange(100).reshape(10, 10))
+            cb[nn][mm] = fig.colorbar(pc, ax=ax[mm, nn], orientation=orient,
+                                      aspect=aspect, extend=extend)
+    fig.draw_without_rendering()
+    # check the extends are right ratio:
+    np.testing.assert_almost_equal(cb[0][1].ax.get_position().height,
+                                   cb[0][0].ax.get_position().height * 0.9,
+                                   decimal=2)
+    # horizontal
+    np.testing.assert_almost_equal(cb[1][1].ax.get_position().width,
+                                   cb[1][0].ax.get_position().width * 0.9,
+                                   decimal=2)
+    # check correct aspect:
+    pos = cb[0][0].ax.get_position(original=False)
+    np.testing.assert_almost_equal(pos.height, pos.width * 20, decimal=2)
+    pos = cb[1][0].ax.get_position(original=False)
+    np.testing.assert_almost_equal(pos.height * 20, pos.width, decimal=2)
+    # check twice as wide if aspect is 10 instead of 20
+    np.testing.assert_almost_equal(
+        cb[0][0].ax.get_position(original=False).width * 2,
+        cb[0][2].ax.get_position(original=False).width, decimal=2)
+    np.testing.assert_almost_equal(
+        cb[1][0].ax.get_position(original=False).height * 2,
+        cb[1][2].ax.get_position(original=False).height, decimal=2)
+
+
+@image_comparison(['proportional_colorbars.png'], remove_text=True,
+                  style='mpl20')
+def test_proportional_colorbars():
+
+    x = y = np.arange(-3.0, 3.01, 0.025)
+    X, Y = np.meshgrid(x, y)
+    Z1 = np.exp(-X**2 - Y**2)
+    Z2 = np.exp(-(X - 1)**2 - (Y - 1)**2)
+    Z = (Z1 - Z2) * 2
+
+    levels = [-1.25, -0.5, -0.125, 0.125, 0.5, 1.25]
+    cmap = mcolors.ListedColormap(
+        ['0.3', '0.5', 'white', 'lightblue', 'steelblue'])
+    cmap.set_under('darkred')
+    cmap.set_over('crimson')
+    norm = mcolors.BoundaryNorm(levels, cmap.N)
+
+    extends = ['neither', 'both']
+    spacings = ['uniform', 'proportional']
+    fig, axs = plt.subplots(2, 2)
+    for i in range(2):
+        for j in range(2):
+            CS3 = axs[i, j].contourf(X, Y, Z, levels, cmap=cmap, norm=norm,
+                                     extend=extends[i])
+            fig.colorbar(CS3, spacing=spacings[j], ax=axs[i, j])
+
+
+def test_negative_boundarynorm():
+    fig, ax = plt.subplots(figsize=(1, 3))
+    cmap = plt.get_cmap("viridis")
+
+    clevs = np.arange(-94, -85)
+    norm = BoundaryNorm(clevs, cmap.N)
+    cb = fig.colorbar(cm.ScalarMappable(cmap=cmap, norm=norm), cax=ax)
+    np.testing.assert_allclose(cb.ax.get_ylim(), [clevs[0], clevs[-1]])
+    np.testing.assert_allclose(cb.ax.get_yticks(), clevs)
+
+    clevs = np.arange(85, 94)
+    norm = BoundaryNorm(clevs, cmap.N)
+    cb = fig.colorbar(cm.ScalarMappable(cmap=cmap, norm=norm), cax=ax)
+    np.testing.assert_allclose(cb.ax.get_ylim(), [clevs[0], clevs[-1]])
+    np.testing.assert_allclose(cb.ax.get_yticks(), clevs)
+
+    clevs = np.arange(-3, 3)
+    norm = BoundaryNorm(clevs, cmap.N)
+    cb = fig.colorbar(cm.ScalarMappable(cmap=cmap, norm=norm), cax=ax)
+    np.testing.assert_allclose(cb.ax.get_ylim(), [clevs[0], clevs[-1]])
+    np.testing.assert_allclose(cb.ax.get_yticks(), clevs)
+
+    clevs = np.arange(-8, 1)
+    norm = BoundaryNorm(clevs, cmap.N)
+    cb = fig.colorbar(cm.ScalarMappable(cmap=cmap, norm=norm), cax=ax)
+    np.testing.assert_allclose(cb.ax.get_ylim(), [clevs[0], clevs[-1]])
+    np.testing.assert_allclose(cb.ax.get_yticks(), clevs)
+
+
+@image_comparison(['nonorm_colorbars.svg'], remove_text=False,
+                  style='mpl20')
+def test_nonorm():
+    plt.rcParams['svg.fonttype'] = 'none'
+    data = [1, 2, 3, 4, 5]
+
+    fig, ax = plt.subplots(figsize=(6, 1))
+    fig.subplots_adjust(bottom=0.5)
+
+    norm = NoNorm(vmin=min(data), vmax=max(data))
+    cmap = cm.get_cmap("viridis", len(data))
+    mappable = cm.ScalarMappable(norm=norm, cmap=cmap)
+    cbar = fig.colorbar(mappable, cax=ax, orientation="horizontal")
+
+
+@image_comparison(['test_boundaries.png'], remove_text=True,
+                  style='mpl20')
+def test_boundaries():
+    np.random.seed(seed=19680808)
+    fig, ax = plt.subplots(figsize=(2, 2))
+    pc = ax.pcolormesh(np.random.randn(10, 10), cmap='RdBu_r')
+    cb = fig.colorbar(pc, ax=ax, boundaries=np.linspace(-3, 3, 7))
